@@ -34,10 +34,12 @@ class BigBed(BigWig):
             columns = []
             lines = data.split("\n")
             for l in lines[3:len(lines)-2]:
-                words = l.split(" ")
-                words = list(filter(None, words))
+                words = l.split(";")
                 if len(words) > 1:
-                    columns.append(words[1])
+                    name = words[0].split(" ")
+                    words = list(filter(None, name))
+                    if len(words) > 1:
+                        columns.append(words[1])
             allColumns = ["chr", "start", "end"]
             allColumns.extend(columns[3:])
             return allColumns
